from django.conf.urls.defaults import *

urlpatterns = patterns('django_banklink.views', 
    (r'^$', 'request', {'description': 'test_payment',
                        'message' : 'tests', 
                        'amount' : '0.05',
                        'currency' : 'LVL',
                        'redirect_to' : 'http://www.done.com/',
                        }),
    (r'^response/$', 'response'),
)



from django.conf import settings 
from django.core.exceptions import ImproperlyConfigured

PRIVATE_KEY =  getattr(settings, 'BANKLINK_PRIVATE_KEY', None)
if not PRIVATE_KEY:
    raise ImproperlyConfigured(u"BANKLINK_PRIVATE_KEY not found in settings")

SND_ID = getattr(settings, 'BANKLINK_SND_ID', None)
if not SND_ID:
    raise ImproperlyConfigured(u"BANKLINK_SND_ID not found in settings")

PUBLIC_KEY = getattr(settings, 'BANKLINK_PUBLIC_KEY', None)
if not PUBLIC_KEY:
    raise ImproperlyConfigured(u"BANKLINK_PUBLIC_KEY not found in settings")

BANKLINK_URL = getattr(settings, 'BANKLINK_REQUEST_URL',
                                'https://ib.swedbank.lv/banklink')





from django.template import RequestContext
from django.core.urlresolvers import reverse 
from django.http import HttpResponseRedirect
from django.http import HttpResponse 

from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404

from django.views.decorators.csrf import csrf_exempt

from django_banklink.forms import PaymentRequest
from django_banklink.utils import verify_signature
from django_banklink.models import Transaction
from django_banklink.signals import transaction_succeeded
from django_banklink.signals import transaction_failed

@csrf_exempt
def response(request):
    if request.method == 'POST':
        data = request.POST
    else: 
        data = request.GET
    if 'VK_MAC' not in data:
        raise Http404("VK_MAC not in request")
    signature_valid = verify_signature(data, data['VK_MAC'])
    if not signature_valid:
        raise Http404("Invalid signature. ")
    transaction = get_object_or_404(Transaction, pk = data['VK_REF'])  
    if data['VK_AUTO'] == 'Y':
        transaction.status = 'C'
        transaction.save()
        transaction_succeeded.send(Transaction, transaction = transaction)
        return HttResponse("request handled, swedbank")
    else:
        if data['VK_SERVICE'] == '1901':
            url = transaction.redirect_on_failure
            transaction.status = 'F'
            transaction.save()
            transaction_failed.send(Transaction, transaction = transaction)
        else:
            url = transaction.redirect_after_success 
        return HttpResponseRedirect(url)

def request(request, description, message, amount, currency, redirect_to):
    if 'HTTP_HOST' not in request.META:
        raise Http404("HTTP/1.1 protocol only, specify Host header")
    protocol = 'https' if request.is_secure() else 'http'
    url = '%s://%s%s' % (protocol, request.META['HTTP_HOST'], reverse(response))
    context = RequestContext(request)
    user = None if request.user.is_anonymous() else request.user 
    context['form'] = PaymentRequest(description = description,
                                     amount = amount,
                                     currency = currency,
                                     redirect_to = redirect_to,
                                     message = message,
                                     user = user)
    return render_to_response("django_banklink/request.html", context)



from django import forms 
from django.core.urlresolvers import reverse 
from django.contrib.sites.models import Site
from django.utils.safestring import SafeUnicode
from django_banklink import settings
from django_banklink.utils import create_signature
from django_banklink.models import Transaction
from warnings import warn
from django_banklink.signals import transaction_started



class PaymentRequest(forms.Form):
    VK_SERVICE = forms.CharField(widget = forms.HiddenInput())
    VK_VERSION = forms.CharField(widget = forms.HiddenInput())
    VK_SND_ID = forms.CharField(widget = forms.HiddenInput())
    VK_STAMP = forms.CharField(widget = forms.HiddenInput())
    VK_AMOUNT = forms.CharField(widget = forms.HiddenInput())
    VK_CURR = forms.CharField(widget = forms.HiddenInput())
    VK_REF = forms.CharField(widget = forms.HiddenInput())
    VK_MSG = forms.CharField(widget = forms.HiddenInput())
    VK_MAC = forms.CharField(widget = forms.HiddenInput(), required = False)
    VK_RETURN = forms.CharField(widget = forms.HiddenInput())
    VK_LANG = forms.CharField(widget = forms.HiddenInput())
    VK_ENCODING = forms.CharField(widget = forms.HiddenInput())
    def __init__(self, *args, **kwargs):
        initial = {} 
        transaction = Transaction()
        transaction.user = kwargs.get('user')
        transaction.description = kwargs.get('message')
        transaction.amount = initial['VK_AMOUNT'] = kwargs.get('amount')
        transaction.currency = initial['VK_CURR'] = kwargs.get('currency', 'LVL')
        transaction.message = initial['VK_MSG'] = kwargs.get('message')
        initial['VK_ENCODING'] = 'UTF-8'
        initial['VK_RETURN'] = "%s%s" % (Site.objects.get_current().domain, reverse('django_banklink.views.response'))
        initial['VK_SERVICE'] = '1002'
        initial['VK_VERSION'] = '008'
        initial['VK_LANG'] = kwargs.get('language', 'LAT')
        initial['VK_SND_ID'] = settings.SND_ID
        transaction.redirect_after_success = kwargs.get('redirect_to')
        transaction.redirect_on_failure = kwargs.get('redirect_on_failure', transaction.redirect_after_success)
        transaction.save()
        transaction_started.send(Transaction, transaction = transaction)
        self.transaction = transaction
        initial['VK_REF'] = transaction.pk 
        initial['VK_STAMP'] = transaction.pk
        super(PaymentRequest, self).__init__(initial, *args)
        if self.is_valid():
            mac = create_signature(self.cleaned_data)
            self.data['VK_MAC'] = mac
            if not self.is_valid():
                raise RuntimeError("signature is invalid")
        else:
            raise RuntimeError("invalid initial data")

    def redirect_html(self):
        """ Hanzanet redirection html"""
        html = u'<form action="%s" method="POST" id="banklink_redirect_url">' % (settings.BANKLINK_URL)
        for field in self:
            html += unicode(field) + u"\n"
        html += u'</form>'
        html += u'''<script type="text/javascript">
                    document.forms['banklink_redirect_url'].submit();
                    </script>'''
        return SafeUnicode(html)
    def submit_button(self, value = u"Apmaksāt"):
        html = u'<form action="%s" method="POST">' % (settings.BANKLINK_URL)
        for field in self:
            html += unicode(field) + u"\n"
        html += '<input type="submit" value="%s" />' % (value)
        html += '</form>'
        return SafeUnicode(html)
    def as_html(self, with_submit = False, id = "banklink_payment_form", submit_value = "submit" ):
        """ return transaction form for redirect to HanzaNet """
        warn("deprecated", DeprecationWarning)
        html = u'<form action="%s" method="POST" id="%s">' % (settings.BANKLINK_URL, id)
        for field in self:
            html += unicode(field) + u"\n"
        if with_submit:
            html += '<input type="submit" value="%s"/>' % (submit_value, )
        html += '</form>'
        return html 



from distutils.core import setup 
setup(
    name = "django-banklink",
    packages = ["django_banklink"],
    version = "0.14",
    description = "django application for Swedbank payment gateway",
    author = "Kristaps Kulis",
    author_email = "kristaps@true-vision.net",
    url = "http://www.github.com/truevision/django_banklink",
    classifiers = [
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Topic :: Office/Business :: Financial',
    ],
    requires = ['M2Crypto (>=0.20)'], 
    )



from M2Crypto import EVP
from M2Crypto import RSA
from M2Crypto import X509

from django_banklink import settings
from base64 import b64encode
from base64 import b64decode 

def get_ordered_request(request):
    def append_if_exists(target, source, value):
        if value in source:
            target.append(source[value])
        return target
    expected_values = ('VK_SERVICE',
                        'VK_VERSION',
                        'VK_SND_ID',
                        'VK_REC_ID',
                        'VK_STAMP',
                        'VK_T_NO',
                        'VK_AMOUNT',
                        'VK_CURR',
                        'VK_REC_ACC',
                        'VK_REC_NAME',
                        'VK_SND_ACC',
                        'VK_SND_NAME',
                        'VK_REF',
                        'VK_MSG',
                        'VK_T_DATE',)

    ordered_request = []
    for value in expected_values:
        ordered_request = append_if_exists(ordered_request, request, value)
    return ordered_request
def request_digest(request):
    """ 
        return request digest in Banklink signature form (see docs for format)
    """
    request = get_ordered_request(request)
    digest = r''
    for value in request:
        
        digest += unicode(len(value)).rjust(3, u'0')
        digest += unicode(value)
    return digest

def create_signature(request):
    """ 
        sign BankLink request in dict format with private_key
    """

    key = EVP.load_key(settings.PRIVATE_KEY)
    digest = request_digest(request)
    key.reset_context(md = 'sha1')
    key.sign_init()
    key.sign_update(digest)
    key_binary = key.sign_final()
    b21 = b64encode(key_binary)
    return b21 

def verify_signature(request, signature):
    """
        verify BankLink reply signature
    """
    signature = unicode(signature)
    cert = X509.load_cert(settings.PUBLIC_KEY)
    pubkey = cert.get_pubkey()
    pubkey.verify_init()
    pubkey.verify_update(request_digest(request))
    if pubkey.verify_final(b64decode(signature)) == 1:
        return True
    else:
        return False
